from typing import Optional
from typing_extensions import override

from anyrun.iterators.base_iterator import BaseIterator
from anyrun.connectors.threat_intelligence.feeds_connector import FeedsConnector


class TaxiiStixFeedsIterator(BaseIterator):
    def __init__(
        self,
        connector: FeedsConnector,
        chunk_size: int = 1,
        collection: str = 'full',
        match_type: Optional[str] = 'indicator',
        match_id: Optional[str] = None,
        match_version: str = 'all',
        match_revoked: bool = False,
        added_after: Optional[str] = None,
        modified_after: Optional[str] = None,
        limit: int = 10000,
        get_delta: bool = False
    ) -> None:
        """
        Returns a list of ANY.RUN Feeds TAXII stix objects according to the specified query parameters

        :param collection: Collection type. Supports: full, ip, url, domain.
        :param match_type: Filter results based on the STIX object types. You can enter multiple values
            separated by commas
        :param match_id: IOC identifier.
        :param match_version: Filter STIX objects by their object version.
        :param match_revoked: Enable or disable receiving revoked feeds in report.
        :param added_after: Receive IOCs after specified date. Example: 2025-04-15.
        :param modified_after: Receive IOCs after specified date. Example: 2025-04-15.
        :param limit: Number of tasks on a page. Default, all IOCs are included.
        :param get_delta: Get only indicators modified since the last request. Works starting from the second request
        :return: The list of feeds in **stix** format
        """
        super().__init__(connector, chunk_size=chunk_size)

        self._query_params = {
            'collection': collection,
            'match_type': match_type,
            'match_id': match_id,
            'match_revoked': match_revoked,
            'match_version': match_version,
            'modified_after': modified_after,
            'added_after': added_after,
            'limit': limit,
            'get_delta': get_delta
        }

        self._taxii_page_id: Optional[str] = None
        self._stop_iteration: bool = False

    @override
    async def _read_next_chunk(self) -> None:
        """ Overrides parent method using TI Feeds requests """
        if self._stop_iteration:
            return

        response = await self._connector.get_taxii_stix_async(
            **self._query_params,
            next_page=self._taxii_page_id
        )

        next_page_id = response.get('next')

        if next_page_id:
            self._taxii_page_id = response.get('next')
        else:
            self._stop_iteration = True

        self._buffer = response.get('objects')
    


from typing import Optional, Union


class RunTimeException(Exception):
    def __init__(self, description: str, code: Optional[Union[str, int]] = None) -> None:
        self._description = description
        self._code = code

    def __str__(self) -> str:
        return (
            f'[AnyRun Exception] '
            f'Status code: {int(self._code) if self._code else "unspecified"}. '
            f'Description: {self._description}'
        )

    @property
    def json(self) -> dict:
        return {
                'description': self._description,
                'code': self._code,
            }

    @property
    def status_code(self) -> Optional[str]:
        return self._code

    @property
    def description(self) -> str:
        return self._description



from uuid import UUID
from typing import Optional, Union

import aiohttp

from anyrun.connectors.sandbox.base_connector import BaseSandboxConnector
from anyrun.utils.config import Config
from anyrun.utils.utility_functions import execute_synchronously


class LinuxConnector(BaseSandboxConnector):
    """
    Provides ANY.RUN TI Yara Lookup endpoints management.
    Uses aiohttp library for the asynchronous calls
    """
    def __init__(
        self,
        api_key: str,
        integration: str = Config.PUBLIC_INTEGRATION,
        trust_env: bool = False,
        verify_ssl: Optional[str] = None,
        proxy: Optional[str] = None,
        proxy_username: Optional[str] = None,
        proxy_password: Optional[str] = None,
        connector: Optional[aiohttp.BaseConnector] = None,
        timeout: int = Config.DEFAULT_REQUEST_TIMEOUT_IN_SECONDS,
        enable_requests: bool = False
    ) -> None:
        """
        :param api_key: ANY.RUN API-KEY in format: API-KEY <token> or Basic token in format: Basic <base64_auth>.
        :param integration: Name of the integration
        :param trust_env: Trust environment settings for proxy configuration
        :param verify_ssl: Enable/disable SSL verification option.
        :param proxy: Proxy url. Example: http://<host>:<port>.
        :param proxy_username: Proxy username.
        :param proxy_password: Proxy password.
        :param connector: A custom aiohttp connector.
        :param timeout: Override the session’s timeout.
        :param enable_requests: Use requests.request to make api calls. May block the event loop.
        """
        super().__init__(
            api_key,
            integration,
            trust_env,
            verify_ssl,
            proxy,
            proxy_username,
            proxy_password,
            connector,
            timeout,
            enable_requests
        )

    def run_file_analysis(
        self,
        file_content: Optional[bytes] = None,
        filename: Optional[str] = None,
        filepath: Optional[str] = None,
        env_os: str = 'ubuntu',
        env_locale: str = 'en-US',
        opt_network_connect: bool = True,
        opt_network_fakenet: bool = False,
        opt_network_tor: bool = False,
        opt_network_geo: str = 'fastest',
        opt_network_mitm: bool = False,
        opt_network_residential_proxy: bool = False,
        opt_network_residential_proxy_geo: str = 'fastest',
        opt_kernel_heavyevasion: bool = False,
        opt_privacy_type: str = 'bylink',
        opt_timeout: int = 60,
        opt_automated_interactivity: bool = True,
        obj_ext_startfolder: str = 'temp',
        obj_ext_cmd: Optional[str] = None,
        run_as_root: bool = False,
        obj_ext_extension: bool = True,
        user_tags: Optional[str] = None,
        task_rerun_uuid: Optional[str] = None
    ) -> Union[UUID, str]:
        """
        Initializes a new file analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_
        Options: file_content and filename have higher priority then filepath option

        :param file_content: File bytes to analyse.
        :param filename: Filename with file extension.
        :param filepath: Absolute path to file. If specified, automatically process file content and filename
        :param env_os: Operation system. Supports: ubuntu, debian
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_startfolder: Start object from. Supports: desktop, home, downloads, temp
        :param obj_ext_cmd: Optional command line.
        :param run_as_root: Run file with superuser privileges
        :param obj_ext_extension: Change extension to valid
        :param user_tags: Append user tags to new analysis. Only characters a-z, A-Z, 0-9, hyphen (-), and comma (,)
            are allowed. Max tag length: 16 characters. Max unique tags per task: 8.
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        return execute_synchronously(
            self.run_file_analysis_async,
            file_content=file_content,
            filename=filename,
            filepath=filepath,
            env_os=env_os,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            obj_ext_startfolder=obj_ext_startfolder,
            obj_ext_cmd=obj_ext_cmd,
            run_as_root=run_as_root,
            obj_ext_extension=obj_ext_extension,
            task_rerun_uuid=task_rerun_uuid,
            user_tags=user_tags
        )

    async def run_file_analysis_async(
        self,
        file_content: Optional[bytes] = None,
        filename: Optional[str] = None,
        filepath: Optional[str] = None,
        env_os: str = 'ubuntu',
        env_locale: str = 'en-US',
        opt_network_connect: bool = True,
        opt_network_fakenet: bool = False,
        opt_network_tor: bool = False,
        opt_network_geo: str = 'fastest',
        opt_network_mitm: bool = False,
        opt_network_residential_proxy: bool = False,
        opt_network_residential_proxy_geo: str = 'fastest',
        opt_kernel_heavyevasion: bool = False,
        opt_privacy_type: str = 'bylink',
        opt_timeout: int = 60,
        opt_automated_interactivity: bool = True,
        obj_ext_startfolder: str = 'temp',
        obj_ext_cmd: Optional[str] = None,
        run_as_root: bool = False,
        obj_ext_extension: bool = True,
        user_tags: Optional[str] = None,
        task_rerun_uuid: Optional[str] = None
    ) -> Union[UUID, str]:
        """
        Initializes a new file analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_
        Options: file_content and filename have higher priority then filepath option

        :param file_content: File bytes to analyse.
        :param filename: Filename with file extension.
        :param filepath: Absolute path to file. If specified, automatically process file content and filename
        :param env_os: Operation system. Supports: ubuntu, debian
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_startfolder: Start object from. Supports: desktop, home, downloads, temp
        :param obj_ext_cmd: Optional command line.
        :param run_as_root: Run file with superuser privileges
        :param obj_ext_extension: Change extension to valid
        :param user_tags: Append user tags to new analysis. Only characters a-z, A-Z, 0-9, hyphen (-), and comma (,)
            are allowed. Max tag length: 16 characters. Max unique tags per task: 8.
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis'
        params = {
            'env_os': 'linux',
            'env_version': '22.04.2' if env_os == 'ubuntu' else '12.2',
            'env_bitness': '64',
            'env_type': 'complete',
            'env_locale': env_locale,
            'opt_network_connect': opt_network_connect,
            'opt_network_fakenet': opt_network_fakenet,
            'opt_network_tor': opt_network_tor,
            'opt_network_geo': opt_network_geo,
            'opt_network_mitm': opt_network_mitm,
            'opt_network_residential_proxy': opt_network_residential_proxy,
            'opt_network_residential_proxy_geo': opt_network_residential_proxy_geo,
            'opt_kernel_heavyevasion': opt_kernel_heavyevasion,
            'opt_privacy_type': opt_privacy_type,
            'opt_timeout': opt_timeout,
            'opt_automated_interactivity': opt_automated_interactivity,
            'obj_ext_startfolder': obj_ext_startfolder,
            'obj_ext_cmd': obj_ext_cmd,
            'obj_force_elevation': None,
            'auto_confirm_uac': None,
            'run_as_root': run_as_root,
            'obj_ext_extension': obj_ext_extension,
            'task_rerun_uuid': task_rerun_uuid,
            'user_tags': user_tags
        }

        if self._enable_requests:
            file_content, filename = await self._get_file_payload(file_content, filename, filepath)
            files = {filename: (filename, file_content)}
            response_data = await self._make_request_async('POST', url, json=params, files=files)
        else:
            body = await self._generate_multipart_request_body(
                file_content=file_content,
                filename=filename,
                filepath=filepath,
                **params
            )
            response_data = await self._make_request_async('POST', url, data=body)
        return response_data.get('data').get('taskid')

    def run_url_analysis(
        self,
        obj_url: str,
        env_os: str = 'ubuntu',
        env_locale: str = 'en-US',
        opt_network_connect: bool = True,
        opt_network_fakenet: bool = False,
        opt_network_tor: bool = False,
        opt_network_geo: str = 'fastest',
        opt_network_mitm: bool = False,
        opt_network_residential_proxy: bool = False,
        opt_network_residential_proxy_geo: str = 'fastest',
        opt_kernel_heavyevasion: bool = False,
        opt_privacy_type: str = 'bylink',
        opt_timeout: int = 60,
        opt_automated_interactivity: bool = True,
        obj_ext_browser: str = 'Google Chrome',
        obj_ext_extension: bool = True,
        user_tags: Optional[str] = None,
        task_rerun_uuid: Optional[str] = None
    ) -> Union[UUID, str]:
        """
        Initializes a new analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_

        :param obj_url: Target URL. Size range 5-512. Example: (http/https)://(your-link)
        :param env_os: Operation system. Supports: ubuntu, debian
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_browser: Browser name. Supports: Google Chrome, Mozilla Firefox
        :param obj_ext_extension: Change extension to valid
        :param user_tags: Append user tags to new analysis. Only characters a-z, A-Z, 0-9, hyphen (-), and comma (,)
            are allowed. Max tag length: 16 characters. Max unique tags per task: 8.
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        return execute_synchronously(
            self.run_url_analysis_async,
            obj_url=obj_url,
            env_os=env_os,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            task_rerun_uuid=task_rerun_uuid,
            obj_ext_browser=obj_ext_browser,
            obj_ext_extension=obj_ext_extension,
            user_tags=user_tags
        )

    async def run_url_analysis_async(
        self,
        obj_url: str,
        env_os: str = 'ubuntu',
        env_locale: str = 'en-US',
        opt_network_connect: bool = True,
        opt_network_fakenet: bool = False,
        opt_network_tor: bool = False,
        opt_network_geo: str = 'fastest',
        opt_network_mitm: bool = False,
        opt_network_residential_proxy: bool = False,
        opt_network_residential_proxy_geo: str = 'fastest',
        opt_kernel_heavyevasion: bool = False,
        opt_privacy_type: str = 'bylink',
        opt_timeout: int = 60,
        opt_automated_interactivity: bool = True,
        obj_ext_browser: str = 'Google Chrome',
        obj_ext_extension: bool = True,
        user_tags: Optional[str] = None,
        task_rerun_uuid: Optional[str] = None
    ) -> Union[UUID, str]:
        """
        Initializes a new analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_

        :param obj_url: Target URL. Size range 5-512. Example: (http/https)://(your-link)
        :param env_os: Operation system. Supports: ubuntu, debian
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_browser: Browser name. Supports: Google Chrome, Mozilla Firefox
        :param obj_ext_extension: Change extension to valid
        :param user_tags: Append user tags to new analysis. Only characters a-z, A-Z, 0-9, hyphen (-), and comma (,)
            are allowed. Max tag length: 16 characters. Max unique tags per task: 8.
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis'

        body = await self._generate_request_body(
            'url',
            obj_url=obj_url,
            env_os='linux',
            env_version='22.04.2' if env_os == 'ubuntu' else '12.2',
            env_bitness='64',
            env_type='complete',
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            task_rerun_uuid=task_rerun_uuid,
            obj_ext_browser=obj_ext_browser,
            obj_ext_extension=obj_ext_extension,
            user_tags=user_tags
        )
        response_data = await self._make_request_async('POST', url, json=body)
        return response_data.get('data').get('taskid')

    def run_download_analysis(
        self,
        obj_url: str,
        env_os: str = 'ubuntu',
        env_locale: str = 'en-US',
        opt_network_connect: bool = True,
        opt_network_fakenet: bool = False,
        opt_network_tor: bool = False,
        opt_network_geo: str = 'fastest',
        opt_network_mitm: bool = False,
        opt_network_residential_proxy: bool = False,
        opt_network_residential_proxy_geo: str = 'fastest',
        opt_kernel_heavyevasion: bool = False,
        opt_privacy_type: str = 'bylink',
        opt_timeout: int = 60,
        opt_automated_interactivity: bool = True,
        obj_ext_startfolder: str = 'temp',
        obj_ext_cmd: Optional[str] = None,
        obj_ext_useragent: Optional[str] = None,
        obj_ext_extension: bool = True,
        opt_privacy_hidesource: bool = False,
        user_tags: Optional[str] = None,
        task_rerun_uuid: Optional[str] = None
    ) -> Union[UUID, str]:
        """
        Initializes a new analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_

        :param obj_url: Target URL. Size range 5-512. Example: (http/https)://(your-link)
        :param env_os: Operation system. Supports: ubuntu, debian
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_startfolder: Start object from. Supports: desktop, home, downloads, temp
        :param obj_ext_cmd: Optional command line.
        :param obj_ext_useragent: User-Agent value.
        :param obj_ext_extension: Change extension to valid
        :param opt_privacy_hidesource: Option for hiding of source URL.
        :param user_tags: Append user tags to new analysis. Only characters a-z, A-Z, 0-9, hyphen (-), and comma (,)
            are allowed. Max tag length: 16 characters. Max unique tags per task: 8.
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        return execute_synchronously(
            self.run_download_analysis_async,
            obj_url=obj_url,
            env_os=env_os,
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            obj_ext_startfolder=obj_ext_startfolder,
            task_rerun_uuid=task_rerun_uuid,
            obj_ext_cmd=obj_ext_cmd,
            obj_ext_useragent=obj_ext_useragent,
            obj_ext_extension=obj_ext_extension,
            opt_privacy_hidesource=opt_privacy_hidesource,
            user_tags=user_tags
        )

    async def run_download_analysis_async(
        self,
        obj_url: str,
        env_os: str = 'ubuntu',
        env_locale: str = 'en-US',
        opt_network_connect: bool = True,
        opt_network_fakenet: bool = False,
        opt_network_tor: bool = False,
        opt_network_geo: str = 'fastest',
        opt_network_mitm: bool = False,
        opt_network_residential_proxy: bool = False,
        opt_network_residential_proxy_geo: str = 'fastest',
        opt_kernel_heavyevasion: bool = False,
        opt_privacy_type: str = 'bylink',
        opt_timeout: int = 60,
        opt_automated_interactivity: bool = True,
        obj_ext_startfolder: str = 'temp',
        obj_ext_cmd: Optional[str] = None,
        obj_ext_useragent: Optional[str] = None,
        obj_ext_extension: bool = True,
        opt_privacy_hidesource: bool = False,
        user_tags: Optional[str] = None,
        task_rerun_uuid: Optional[str] = None
    ) -> Union[UUID, str]:
        """
        Initializes a new analysis according to the specified parameters
        You can find extended documentation `here <https://any.run/api-documentation/#api-Analysis-PostAnalysis>`_

        :param obj_url: Target URL. Size range 5-512. Example: (http/https)://(your-link)
        :param env_os: Operation system. Supports: ubuntu, debian
        :param env_locale: Operation system's language. Use locale identifier or country name (Ex: "en-US" or "Brazil").
            Case insensitive.
        :param opt_network_connect: Network connection state
        :param opt_network_fakenet: FakeNet feature status
        :param opt_network_tor: TOR using
        :param opt_network_geo: Tor geo location option. Example: US, AU
        :param opt_network_mitm: HTTPS MITM proxy option.
        :param opt_network_residential_proxy: Residential proxy using
        :param opt_network_residential_proxy_geo: Residential proxy geo location option. Example: US, AU
        :param opt_kernel_heavyevasion: Heavy evasion option
        :param opt_privacy_type: Privacy settings. Supports: public, bylink, owner, byteam
        :param opt_timeout: Timeout option. Size range: 10-660
        :param opt_automated_interactivity: Automated Interactivity (ML) option
        :param obj_ext_startfolder: Start object from. Supports: desktop, home, downloads, temp
        :param obj_ext_cmd: Optional command line.
        :param obj_ext_useragent: User-Agent value.
        :param obj_ext_extension: Change extension to valid
        :param opt_privacy_hidesource: Option for hiding of source URL.
        :param user_tags: Append user tags to new analysis. Only characters a-z, A-Z, 0-9, hyphen (-), and comma (,)
            are allowed. Max tag length: 16 characters. Max unique tags per task: 8.
        :param task_rerun_uuid: Completed task identifier. Re-runs an existent task if uuid is specified. You can re-run
            task with new parameters
        :return: Task uuid
        """
        url = f'{Config.ANY_RUN_API_URL}/analysis'

        body = await self._generate_request_body(
            'download',
            obj_url=obj_url,
            env_os='linux',
            env_version='22.04.2' if env_os == 'ubuntu' else '12.2',
            env_bitness='64',
            env_type='complete',
            env_locale=env_locale,
            opt_network_connect=opt_network_connect,
            opt_network_fakenet=opt_network_fakenet,
            opt_network_tor=opt_network_tor,
            opt_network_geo=opt_network_geo,
            opt_network_mitm=opt_network_mitm,
            opt_network_residential_proxy=opt_network_residential_proxy,
            opt_network_residential_proxy_geo=opt_network_residential_proxy_geo,
            opt_kernel_heavyevasion=opt_kernel_heavyevasion,
            opt_privacy_type=opt_privacy_type,
            opt_timeout=opt_timeout,
            opt_automated_interactivity=opt_automated_interactivity,
            obj_ext_startfolder=obj_ext_startfolder,
            task_rerun_uuid=task_rerun_uuid,
            obj_ext_cmd=obj_ext_cmd,
            obj_ext_useragent=obj_ext_useragent,
            obj_ext_extension=obj_ext_extension,
            opt_privacy_hidesource=opt_privacy_hidesource,
            user_tags=user_tags
        )

        response_data = await self._make_request_async('POST', url, json=body)
        return response_data.get('data').get('taskid')



from http import HTTPStatus
from typing import Optional, Union, Any
from typing_extensions import Self
from abc import abstractmethod
from traceback import format_exc

import aiohttp
import asyncio
import requests

from aiohttp import BasicAuth
from urllib3 import disable_warnings
from urllib3.exceptions import InsecureRequestWarning

from anyrun.utils.config import Config
from anyrun.utils.exceptions import RunTimeException
from anyrun.utils.utility_functions import execute_synchronously, get_running_loop

disable_warnings(InsecureRequestWarning)

class AnyRunConnector:

    def __init__(
        self,
        api_key: str,
        integration: str = Config.PUBLIC_INTEGRATION,
        trust_env: bool = False,
        verify_ssl: Optional[str] = None,
        proxy: Optional[str] = None,
        proxy_username: Optional[str] = None,
        proxy_password: Optional[str] = None,
        connector: Optional[aiohttp.BaseConnector] = None,
        timeout: int = Config.DEFAULT_REQUEST_TIMEOUT_IN_SECONDS,
        enable_requests: bool = False
    ) -> None:
        """
        :param api_key: ANY.RUN API-KEY in format: API-KEY <token> or Basic token in format: Basic <base64_auth>.
        :param integration: Name of the integration.
        :param trust_env: Trust environment settings for proxy configuration.
        :param verify_ssl: Enable/disable SSL verification option.
        :param proxy: Proxy url. Example: https://<host>:<port>.
        :param proxy_username: Proxy username.
        :param proxy_password: Proxy password.
        :param connector: A custom aiohttp connector.
        :param timeout: Override the session’s timeout.
        :param enable_requests: Use requests.request to make api calls. May block the event loop.
        """
        self._proxy = proxy
        self._proxy_username = proxy_username
        self._proxy_password = proxy_password
        self._trust_env = trust_env
        self._connector = connector
        self._timeout = timeout
        self._enable_requests = enable_requests
        self._verify_ssl = verify_ssl
        self._session: Optional[aiohttp.ClientSession] = None

        self._api_key_validator(api_key)
        self._setup_connector()
        self._setup_headers(api_key, integration)

        self._response_headers: dict[str, Any] = dict()

    def __enter__(self) -> Self:
        execute_synchronously(self._open_session)
        return self

    def __exit__(self, item_type, value, traceback) -> None:
        execute_synchronously(self._close_session)

    async def __aenter__(self) -> Self:
        await self._open_session()
        return self

    async def __aexit__(self, item_type, value, traceback) -> None:
        await self._close_session()

    def check_proxy(self) -> dict:
        """
        Executes test proxy request to google.com

        :returns: Verification status
        """
        return execute_synchronously(self.check_proxy_async)

    async def check_proxy_async(self) -> dict:
        """"
        Executes test proxy request to google.com

        :returns: Verification status
        """
        try:
            await self._make_request_async('GET', 'https://google.com', request_timeout=5, parse_response=False)
        except (aiohttp.ClientError, requests.RequestException, OSError) as exception:
            raise RunTimeException('The proxy request failed. Check the proxy settings are correct') from exception
        return {'status': 'ok', 'description': 'Successful proxy verification'}

    async def _make_request_async(
        self,
        method: str,
        url: str,
        json: Optional[dict] = None,
        data: Union[dict, aiohttp.MultipartWriter, None] = None,
        files: Optional[dict[str, tuple[str, bytes]]] = None,
        parse_response: bool = True,
        request_timeout: Optional[int] = None
    ) -> Union[dict, list[dict], aiohttp.ClientResponse, requests.Response]:
        """
        Provides async interface for making any request

        :param method: HTTP method
        :param url: Request url
        :param json: Request json
        :param data: Request data
        :param files: Request files (only for the requests package)
        :param parse_response: Enable/disable API response parsing. If enabled, returns response.json() object dict
            else aiohttp.ClientResponse instance
        :param request_timeout: HTTP Request timeout
        :return: Api response
        :raises RunTimeException: If the connector was executed outside the context manager
        """
        try:
            if self._enable_requests:
                response: requests.Response = requests.request(
                    method,
                    url,
                    headers=self._headers,
                    json=json,
                    params=data,
                    files=files,
                    verify=self._verify_ssl,
                    proxies=self._generate_proxy_config() if self._proxy else None,
                    timeout=request_timeout
                )
            else:
                response: aiohttp.ClientResponse = await self._session.request(
                    method,
                    url,
                    json=json,
                    data=data,
                    ssl=True if self._verify_ssl else False,
                    timeout=request_timeout
                )

            self._response_headers = response.headers

            if parse_response:
                response_data = response.json() if self._enable_requests else await response.json()
                return await self._check_response_status(
                    response_data,
                    response.status_code if self._enable_requests else response.status
                )
            return response

        except AttributeError:
            raise RunTimeException('The connector object must be executed using the context manager')
        except (aiohttp.ClientError, requests.RequestException, OSError) as exception:
            raise RunTimeException(f'Connection error: {format_exc(exception)}')

    def _setup_connector(self) -> None:
        if not self._connector and self._verify_ssl:
            event_loop = get_running_loop()
            asyncio.set_event_loop(event_loop)
            self._connector = aiohttp.TCPConnector(ssl=self._verify_ssl, loop=event_loop)

    def _setup_headers(self, api_key: str, integration: str) -> None:
        self._headers = {
            'Authorization': api_key,
            'x-anyrun-connector': integration,
            'x-anyrun-sdk': Config.SDK_VERSION
        }

    async def _open_session(self) -> None:
        if not self._session:
            self._session = aiohttp.ClientSession(
                trust_env=self._trust_env,
                connector=self._connector,
                proxy=self._proxy,
                auth=BasicAuth(self._proxy_username, self._proxy_password) if self._proxy_username else None,
                timeout=aiohttp.ClientTimeout(total=self._timeout),
                headers=self._headers
            )

    async def _close_session(self) -> None:
        if self._session:
            await self._session.close()
            self._session = None

    def _generate_proxy_config(self) -> Optional[dict[str, str]]:
        """
        Generates proxies dict using received proxy string

        :return: Proxy dict
        """
        if self._proxy:
            protocol = self._proxy.split(':')[0]

            if self._proxy_username and self._proxy_password:
                connection = self._proxy[self._proxy.index('//') + 2:]
                return {protocol: f'{protocol}://{self._proxy_username}:{self._proxy_password}@{connection}'}

            return {protocol: self._proxy}

    @abstractmethod
    def check_authorization(self) -> dict:
        """
        Makes a request to check the validity of the API key.
        The request does not consume the license

        :return: Verification status
        """
        pass

    @abstractmethod
    async def check_authorization_async(self) -> dict:
        """
        Makes a request to check the validity of the API key.
        The request does not consume the license

        :return: Verification status
        """
        pass

    @staticmethod
    async def _check_response_status(response_data: dict, status: int) -> dict:
        """
        Process ANY.RUN endpoint response.

        Returns a dictionary with an explanation of the error if the response status code is not equal **OK**

        :param response_data: API response
        :return: The collection of IOCs
        :raises RunTimeException: If status code 200 is not received
        """
        if status in (HTTPStatus.OK, HTTPStatus.CREATED, HTTPStatus.ACCEPTED):
            return response_data

        raise RunTimeException(
            response_data.get('message') or response_data.get('description'),
            status or HTTPStatus.BAD_REQUEST
        )

    @staticmethod
    def _api_key_validator(api_key: str) -> None:
        """
        Checks if API key format is valid

        :param api_key: ANY.RUN API-KEY in format: API-KEY <token> or Basic token in format: Basic <base64_auth>.
        :raises RunTimeException: If API key format is not valid
        """
        if not isinstance(api_key, str):
            raise RunTimeException('The ANY.RUN api key must be a valid string')
